@@ -13,7 +13,7 @@ from twisted.internet import reactor
 from gnutls.constants import *
 from gnutls.crypto import *
 from gnutls.errors import *
-from gnutls.interfaces.twisted import X509Credentials
+from gnutls.interfaces.twisted import TLSContext, X509Credentials
 
 class EchoProtocol(LineOnlyReceiver):
 
@@ -46,7 +46,8 @@ ca = X509Certificate(open(certs_path + '/ca.pem').read())
 crl = X509CRL(open(certs_path + '/crl.pem').read())
 cred = X509Credentials(cert, key, [ca])
 cred.verify_peer = True
+context = TLSContext(cred, session_parameters="NORMAL:-COMP-ALL:+COMP-DEFLATE:+COMP-NULL")
 
-reactor.connectTLS('localhost', 10000, EchoFactory(), cred)
+reactor.connectTLS('localhost', 10000, EchoFactory(), context)
 reactor.run()
 
