@@ -1,9 +1,7 @@
-# Copyright (C) 2007-2008 AG Projects. See LICENSE for details.
-#
 
 """GNUTLS connection support"""
 
-__all__ = ['X509Credentials', 'ClientSession', 'ServerSession', 'ServerSessionFactory']
+__all__ = ['X509Credentials', 'TLSContext', 'ClientSession', 'ServerSession', 'ServerSessionFactory']
 
 from time import time
 from socket import SHUT_RDWR as SOCKET_SHUT_RDWR
@@ -24,20 +22,21 @@ from gnutls.library.constants import GNUTLS_A_UNKNOWN_CA, GNUTLS_A_INSUFFICIENT_
 from gnutls.library.constants import GNUTLS_A_CERTIFICATE_EXPIRED, GNUTLS_A_CERTIFICATE_REVOKED
 from gnutls.library.constants import GNUTLS_NAME_DNS
 from gnutls.library.types     import gnutls_certificate_credentials_t, gnutls_session_t, gnutls_x509_crt_t
-from gnutls.library.types     import gnutls_certificate_server_retrieve_function
+from gnutls.library.types     import gnutls_certificate_retrieve_function
+from gnutls.library.types     import gnutls_priority_t
 from gnutls.library.functions import *
 
 
-@gnutls_certificate_server_retrieve_function
-def _retrieve_server_certificate(c_session, retr_st):
+@gnutls_certificate_retrieve_function
+def _retrieve_certificate(c_session, req_ca_dn, nreqs, pk_algos, pk_algos_length, retr_st):
     session = PyObj_FromPtr(gnutls_session_get_ptr(c_session))
     identity = session.credentials.select_server_identity(session)
-    retr_st.contents.type = GNUTLS_CRT_X509
     retr_st.contents.deinit_all = 0
     if identity is None:
         retr_st.contents.ncerts = 0
     else:
         retr_st.contents.ncerts = 1
+        retr_st.contents.cert_type = GNUTLS_CRT_X509
         retr_st.contents.cert.x509.contents = identity.cert._c_object
         retr_st.contents.key.x509 = identity.key._c_object
     return 0
@@ -69,12 +68,6 @@ class _ServerNameIdentities(dict):
 
 
 class X509Credentials(object):
-    DH_BITS  = 1024
-    RSA_BITS = 1024
-
-    dh_params  = None
-    rsa_params = None
-
     def __new__(cls, *args, **kwargs):
         c_object = gnutls_certificate_credentials_t()
         gnutls_certificate_allocate_credentials(byref(c_object))
@@ -91,7 +84,7 @@ class X509Credentials(object):
             gnutls_certificate_set_x509_key(self._c_object, byref(cert._c_object), 1, key._c_object)
         elif (cert, key) != (None, None):
             raise ValueError("Specify neither or both the certificate and private key")
-        gnutls_certificate_server_set_retrieve_function(self._c_object, _retrieve_server_certificate)
+        gnutls_certificate_set_retrieve_function(self._c_object, _retrieve_certificate)
         self._max_depth = 5
         self._max_bits  = 8200
         self._type = CRED_CERTIFICATE
@@ -104,7 +97,6 @@ class X509Credentials(object):
         self.server_name_identities = _ServerNameIdentities(identities)
         if cert and key:
             self.server_name_identities.add(X509Identity(cert, key))
-        self.session_params = SessionParams(self._type)
 
     def __del__(self):
         self.__deinit(self._c_object)
@@ -119,16 +111,6 @@ class X509Credentials(object):
             gnutls_certificate_set_x509_trust(self._c_object, cast(byref(ca_list), POINTER(gnutls_x509_crt_t)), size)
             self._trusted = self._trusted + tuple(trusted)
 
-    def generate_dh_params(self, bits=DH_BITS):
-        reference = self.dh_params ## keep a reference to preserve it until replaced
-        X509Credentials.dh_params  = DHParams(bits)
-        del reference
-
-    def generate_rsa_params(self, bits=RSA_BITS):
-        reference = self.rsa_params ## keep a reference to preserve it until replaced
-        X509Credentials.rsa_params = RSAParams(bits)
-        del reference
-
     # Properties
 
     @property
@@ -197,66 +179,25 @@ class X509Credentials(object):
             return None
 
 
-class SessionParams(object):
-    _default_kx_algorithms = {
-        CRED_CERTIFICATE: (KX_RSA, KX_DHE_DSS, KX_DHE_RSA),
-        CRED_ANON: (KX_ANON_DH,)}
-    _all_kx_algorithms = {
-        CRED_CERTIFICATE: set((KX_RSA, KX_DHE_DSS, KX_DHE_RSA, KX_RSA_EXPORT)),
-        CRED_ANON: set((KX_ANON_DH,))}
+class TLSContext(object):
+    def __init__(self, credentials, session_parameters=None):
+        self.credentials = credentials
+        self.session_parameters = session_parameters
 
-    def __new__(cls, credentials_type):
-        if credentials_type not in cls._default_kx_algorithms:
-            raise TypeError("Unknown credentials type: %r" % credentials_type)
-        return object.__new__(cls)
+    @property
+    def session_parameters(self):
+        return self.__dict__.get('session_parameters')
 
-    def __init__(self, credentials_type):
-        self._credentials_type = credentials_type
-        self._protocols = (PROTO_TLS1_1, PROTO_TLS1_0, PROTO_SSL3)
-        self._kx_algorithms = self._default_kx_algorithms[credentials_type]
-        self._ciphers = (CIPHER_AES_128_CBC, CIPHER_3DES_CBC, CIPHER_ARCFOUR_128)
-        self._mac_algorithms = (MAC_SHA1, MAC_MD5, MAC_RMD160)
-        self._compressions = (COMP_NULL,)
-
-    def _get_protocols(self):
-        return self._protocols
-    def _set_protocols(self, protocols):
-        self._protocols = ProtocolListValidator(protocols)
-    protocols = property(_get_protocols, _set_protocols)
-    del _get_protocols, _set_protocols
-
-    def _get_kx_algorithms(self):
-        return self._kx_algorithms
-    def _set_kx_algorithms(self, algorithms):
-        cred_type = self._credentials_type
-        algorithms = KeyExchangeListValidator(algorithms)
-        invalid = set(algorithms) - self._all_kx_algorithms[cred_type]
-        if invalid:
-            raise ValueError("Cannot specify %r with %r credentials" % (list(invalid), cred_type))
-        self._kx_algorithms = algorithms
-    kx_algorithms = property(_get_kx_algorithms, _set_kx_algorithms)
-    del _get_kx_algorithms, _set_kx_algorithms
-
-    def _get_ciphers(self):
-        return self._ciphers
-    def _set_ciphers(self, ciphers):
-        self._ciphers = CipherListValidator(ciphers)
-    ciphers = property(_get_ciphers, _set_ciphers)
-    del _get_ciphers, _set_ciphers
-
-    def _get_mac_algorithms(self):
-        return self._mac_algorithms
-    def _set_mac_algorithms(self, algorithms):
-        self._mac_algorithms = MACListValidator(algorithms)
-    mac_algorithms = property(_get_mac_algorithms, _set_mac_algorithms)
-    del _get_mac_algorithms, _set_mac_algorithms
-
-    def _get_compressions(self):
-        return self._compressions
-    def _set_compressions(self, compressions):
-        self._compressions = CompressionListValidator(compressions)
-    compressions = property(_get_compressions, _set_compressions)
-    del _get_compressions, _set_compressions
+    @session_parameters.setter
+    def session_parameters(self, value):
+        priority = gnutls_priority_t()
+        try:
+            gnutls_priority_init(byref(priority), value, None)
+        except GNUTLSError:
+            raise ValueError("invalid session parameters: %s" % value)
+        else:
+            gnutls_priority_deinit(priority)
+        self.__dict__['session_parameters'] = value
 
 
 class Session(object):
@@ -273,17 +214,16 @@ class Session(object):
         instance._c_object = gnutls_session_t()
         return instance
 
-    def __init__(self, socket, credentials):
+    def __init__(self, socket, context):
         gnutls_init(byref(self._c_object), self.session_type)
         ## Store a pointer to self on the C session
         gnutls_session_set_ptr(self._c_object, id(self))
         gnutls_set_default_priority(self._c_object)
-        # gnutls_dh_set_prime_bits(session, DH_BITS)?
+        gnutls_priority_set_direct(self._c_object, context.session_parameters, None)
         gnutls_transport_set_ptr(self._c_object, socket.fileno())
         gnutls_handshake_set_private_extensions(self._c_object, 1)
         self.socket = socket
-        self.credentials = credentials
-        self._update_params()
+        self.credentials = context.credentials
 
     def __del__(self):
         self.__deinit(self._c_object)
@@ -353,24 +293,9 @@ class Session(object):
 
     # Session methods
 
-    def _update_params(self):
-        """Update the priorities of the session params using the credentials."""
-        def c_priority_list(priorities):
-            size = len(priorities) + 1
-            return (c_int * size)(*priorities)
-        session_params = self.credentials.session_params
-        # protocol order in the priority list is irrelevant (it always uses newer protocols first)
-        # the protocol list only specifies what protocols are to be enabled.
-        gnutls_protocol_set_priority(self._c_object, c_priority_list(session_params.protocols))
-        gnutls_kx_set_priority(self._c_object, c_priority_list(session_params.kx_algorithms))
-        gnutls_cipher_set_priority(self._c_object, c_priority_list(session_params.ciphers))
-        gnutls_mac_set_priority(self._c_object, c_priority_list(session_params.mac_algorithms))
-        gnutls_compression_set_priority(self._c_object, c_priority_list(session_params.compressions))
-
     def handshake(self):
         gnutls_handshake(self._c_object)
 
-    #@method_args((basestring, buffer))
     def send(self, data):
         data = str(data)
         if not data:
@@ -428,8 +353,8 @@ class Session(object):
 class ClientSession(Session):
     session_type = GNUTLS_CLIENT
 
-    def __init__(self, socket, credentials, server_name=None):
-        Session.__init__(self, socket, credentials)
+    def __init__(self, socket, context, server_name=None):
+        Session.__init__(self, socket, context)
         self._server_name = None
         if server_name is not None:
             self.server_name = server_name
@@ -447,8 +372,8 @@ class ClientSession(Session):
 class ServerSession(Session):
     session_type = GNUTLS_SERVER
 
-    def __init__(self, socket, credentials):
-        Session.__init__(self, socket, credentials)
+    def __init__(self, socket, context):
+        Session.__init__(self, socket, context)
         gnutls_certificate_server_set_request(self._c_object, CERT_REQUEST)
 
     @property
@@ -473,11 +398,11 @@ class ServerSession(Session):
 
 class ServerSessionFactory(object):
 
-    def __init__(self, socket, credentials, session_class=ServerSession):
+    def __init__(self, socket, context, session_class=ServerSession):
         if not issubclass(session_class, ServerSession):
             raise TypeError, "session_class must be a subclass of ServerSession"
         self.socket = socket
-        self.credentials = credentials
+        self.context = context
         self.session_class = session_class
 
     def __getattr__(self, name):
@@ -492,7 +417,7 @@ class ServerSessionFactory(object):
 
     def accept(self):
         new_sock, address = self.socket.accept()
-        session = self.session_class(new_sock, self.credentials)
+        session = self.session_class(new_sock, self.context)
         return (session, address)
 
     def shutdown(self, how=SOCKET_SHUT_RDWR):
